package com.sun.xml.internal.ws.addressing;

import com.sun.istack.internal.Nullable;
import org.w3c.dom.NamedNodeMap;
import java.util.Map;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import java.util.ArrayList;
import javax.xml.namespace.QName;
import java.util.HashMap;
import javax.xml.transform.dom.DOMResult;
import org.w3c.dom.NodeList;
import java.util.Iterator;
import com.sun.xml.internal.stream.buffer.XMLStreamBufferSource;
import javax.xml.stream.XMLStreamException;
import com.sun.xml.internal.ws.wsdl.parser.WSDLConstants;
import com.sun.xml.internal.ws.addressing.v200408.MemberSubmissionAddressingConstants;
import com.sun.xml.internal.ws.util.DOMUtil;
import org.w3c.dom.Element;
import com.sun.xml.internal.ws.api.addressing.AddressingVersion;
import com.sun.xml.internal.stream.buffer.stax.StreamWriterBufferCreator;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.developer.MemberSubmissionEndpointReference;
import javax.xml.ws.wsaddressing.W3CEndpointReference;
import com.sun.istack.internal.NotNull;
import javax.xml.ws.EndpointReference;

public class EndpointReferenceUtil
{
    private static boolean w3cMetadataWritten;
    
    public EndpointReferenceUtil() {
    }
    
    public static <T extends EndpointReference> T transform(Class<T> clazz, @NotNull EndpointReference epr) {
        assert epr != null;
        if (clazz.isAssignableFrom(W3CEndpointReference.class)) {
            if ((epr instanceof W3CEndpointReference)) {
                return epr;
            }
            if ((epr instanceof MemberSubmissionEndpointReference)) {
                return toW3CEpr((MemberSubmissionEndpointReference)epr);
            }
        }
        else if (clazz.isAssignableFrom(MemberSubmissionEndpointReference.class)) {
            if ((epr instanceof W3CEndpointReference)) {
                return toMSEpr((W3CEndpointReference)epr);
            }
            if ((epr instanceof MemberSubmissionEndpointReference)) {
                return epr;
            }
        }
        throw new WebServiceException("Unknwon EndpointReference: " + epr.getClass());
    }
    
    private static W3CEndpointReference toW3CEpr(MemberSubmissionEndpointReference msEpr) {
        StreamWriterBufferCreator writer = new StreamWriterBufferCreator();
        EndpointReferenceUtil.w3cMetadataWritten = false;
        try {
            writer.writeStartDocument();
            writer.writeStartElement(AddressingVersion.W3C.getPrefix(), "EndpointReference", AddressingVersion.W3C.nsUri);
            writer.writeNamespace(AddressingVersion.W3C.getPrefix(), AddressingVersion.W3C.nsUri);
            writer.writeStartElement(AddressingVersion.W3C.getPrefix(), AddressingVersion.W3C.eprType.address, AddressingVersion.W3C.nsUri);
            writer.writeCharacters(msEpr.addr.uri);
            writer.writeEndElement();
            if ((msEpr.referenceProperties != null && msEpr.referenceProperties.elements.size() > 0) || (msEpr.referenceParameters != null && msEpr.referenceParameters.elements.size() > 0)) {
                writer.writeStartElement(AddressingVersion.W3C.getPrefix(), "ReferenceParameters", AddressingVersion.W3C.nsUri);
                if (msEpr.referenceProperties != null) {
                    for (Element e : msEpr.referenceProperties.elements) {
                        DOMUtil.serializeNode(e, writer);
                    }
                }
                if (msEpr.referenceParameters != null) {
                    for (Element e : msEpr.referenceParameters.elements) {
                        DOMUtil.serializeNode(e, writer);
                    }
                }
                writer.writeEndElement();
            }
            Element wsdlElement = null;
            if (msEpr.elements != null && msEpr.elements.size() > 0) {
                for (Element e2 : msEpr.elements) {
                    if (e2.getNamespaceURI().equals(MemberSubmissionAddressingConstants.MEX_METADATA.getNamespaceURI()) && e2.getLocalName().equals(MemberSubmissionAddressingConstants.MEX_METADATA.getLocalPart())) {
                        NodeList nl = e2.getElementsByTagNameNS("http://schemas.xmlsoap.org/wsdl/", WSDLConstants.QNAME_DEFINITIONS.getLocalPart());
                        if (nl != null) {
                            wsdlElement = (Element)nl.item(0);
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
            if (wsdlElement != null) {
                DOMUtil.serializeNode(wsdlElement, writer);
            }
            if (EndpointReferenceUtil.w3cMetadataWritten) {
                writer.writeEndElement();
            }
            if (msEpr.elements != null && msEpr.elements.size() > 0) {
                for (Element e2 : msEpr.elements) {
                    if (e2.getNamespaceURI().equals("http://schemas.xmlsoap.org/wsdl/") && !e2.getLocalName().equals(WSDLConstants.QNAME_DEFINITIONS.getLocalPart())) {}
                    DOMUtil.serializeNode(e2, writer);
                }
            }
            writer.writeEndElement();
            writer.writeEndDocument();
            writer.flush();
        }
        catch (XMLStreamException e3) {
            throw new WebServiceException(e3);
        }
        return new W3CEndpointReference(new XMLStreamBufferSource(writer.getXMLStreamBuffer()));
    }
    
    private static MemberSubmissionEndpointReference toMSEpr(W3CEndpointReference w3cEpr) {
        DOMResult result = new DOMResult();
        w3cEpr.writeTo(result);
        Node eprNode = result.getNode();
        Element e = DOMUtil.getFirstElementChild(eprNode);
        if (e == null) {
            return null;
        }
        MemberSubmissionEndpointReference msEpr = new MemberSubmissionEndpointReference();
        NodeList nodes = e.getChildNodes();
        for (int i = 0; i < nodes.getLength(); ++i) {
            if (nodes.item(i).getNodeType() == 1) {
                Element child = (Element)nodes.item(i);
                if (child.getNamespaceURI().equals(AddressingVersion.W3C.nsUri) && child.getLocalName().equals(AddressingVersion.W3C.eprType.address)) {
                    if (msEpr.addr == null) {
                        msEpr.addr = new MemberSubmissionEndpointReference.Address();
                    }
                    msEpr.addr.uri = XmlUtil.getTextForNode(child);
                }
                else if (child.getNamespaceURI().equals(AddressingVersion.W3C.nsUri) && child.getLocalName().equals("ReferenceParameters")) {
                    NodeList refParams = child.getChildNodes();
                    for (int k = 0; k < refParams.getLength(); ++k) {
                        if (refParams.item(k).getNodeType() == 1) {
                            if (msEpr.referenceParameters == null) {
                                msEpr.referenceParameters = new MemberSubmissionEndpointReference.Elements();
                                msEpr.referenceParameters.elements = new ArrayList();
                            }
                            msEpr.referenceParameters.elements.add((Node)refParams.item(k));
                        }
                    }
                }
                else if (child.getNamespaceURI().equals(AddressingVersion.W3C.nsUri) && child.getLocalName().equals(AddressingVersion.W3C.eprType.wsdlMetadata.getLocalPart())) {
                    NodeList metadata = child.getChildNodes();
                    String wsdlLocation = child.getAttributeNS("http://www.w3.org/ns/wsdl-instance", "wsdlLocation");
                    Element wsdlDefinitions = null;
                    for (int j = 0; j < metadata.getLength(); ++j) {
                        Node node = metadata.item(j);
                        if (node.getNodeType() == 1) {
                            Element elm = (Element)node;
                            if ((elm.getNamespaceURI().equals(AddressingVersion.W3C.wsdlNsUri) || elm.getNamespaceURI().equals("http://www.w3.org/2007/05/addressing/metadata")) && elm.getLocalName().equals(AddressingVersion.W3C.eprType.serviceName)) {
                                msEpr.serviceName = new MemberSubmissionEndpointReference.ServiceNameType();
                                msEpr.serviceName.portName = elm.getAttribute(AddressingVersion.W3C.eprType.portName);
                                String service = elm.getTextContent();
                                String prefix2 = XmlUtil.getPrefix(service);
                                String name = XmlUtil.getLocalPart(service);
                                if (name != null) {
                                    if (prefix2 != null) {
                                        String ns2 = elm.lookupNamespaceURI(prefix2);
                                        if (ns2 != null) {
                                            msEpr.serviceName.name = new QName(ns2, name, prefix2);
                                        }
                                    }
                                    else {
                                        msEpr.serviceName.name = new QName(null, name);
                                    }
                                    msEpr.serviceName.attributes = getAttributes(elm);
                                }
                            }
                            else if ((elm.getNamespaceURI().equals(AddressingVersion.W3C.wsdlNsUri) || elm.getNamespaceURI().equals("http://www.w3.org/2007/05/addressing/metadata")) && elm.getLocalName().equals(AddressingVersion.W3C.eprType.portTypeName)) {
                                msEpr.portTypeName = new MemberSubmissionEndpointReference.AttributedQName();
                                String portType = elm.getTextContent();
                                String prefix2 = XmlUtil.getPrefix(portType);
                                String name = XmlUtil.getLocalPart(portType);
                                if (name != null) {
                                    if (prefix2 != null) {
                                        String ns2 = elm.lookupNamespaceURI(prefix2);
                                        if (ns2 != null) {
                                            msEpr.portTypeName.name = new QName(ns2, name, prefix2);
                                        }
                                    }
                                    else {
                                        msEpr.portTypeName.name = new QName(null, name);
                                    }
                                    msEpr.portTypeName.attributes = getAttributes(elm);
                                }
                            }
                            else if (elm.getNamespaceURI().equals("http://schemas.xmlsoap.org/wsdl/") && elm.getLocalName().equals(WSDLConstants.QNAME_DEFINITIONS.getLocalPart())) {
                                wsdlDefinitions = elm;
                            }
                            else {
                                if (msEpr.elements == null) {
                                    msEpr.elements = new ArrayList();
                                }
                                msEpr.elements.add(elm);
                            }
                        }
                    }
                    Document doc = DOMUtil.createDom();
                    Element mexEl = doc.createElementNS(MemberSubmissionAddressingConstants.MEX_METADATA.getNamespaceURI(), MemberSubmissionAddressingConstants.MEX_METADATA.getPrefix() + ":" + MemberSubmissionAddressingConstants.MEX_METADATA.getLocalPart());
                    Element metadataEl = doc.createElementNS(MemberSubmissionAddressingConstants.MEX_METADATA_SECTION.getNamespaceURI(), MemberSubmissionAddressingConstants.MEX_METADATA_SECTION.getPrefix() + ":" + MemberSubmissionAddressingConstants.MEX_METADATA_SECTION.getLocalPart());
                    metadataEl.setAttribute("Dialect", "http://schemas.xmlsoap.org/wsdl/");
                    if (wsdlDefinitions == null && wsdlLocation != null && !wsdlLocation.equals("")) {
                        wsdlLocation = wsdlLocation.trim();
                        String wsdlTns = wsdlLocation.substring(0, wsdlLocation.indexOf(32));
                        wsdlLocation = wsdlLocation.substring(wsdlLocation.indexOf(32) + 1);
                        Element wsdlEl = doc.createElementNS("http://schemas.xmlsoap.org/wsdl/", "wsdl:" + WSDLConstants.QNAME_DEFINITIONS.getLocalPart());
                        Element wsdlImportEl = doc.createElementNS("http://schemas.xmlsoap.org/wsdl/", "wsdl:" + WSDLConstants.QNAME_IMPORT.getLocalPart());
                        wsdlImportEl.setAttribute("namespace", wsdlTns);
                        wsdlImportEl.setAttribute("location", wsdlLocation);
                        wsdlEl.appendChild(wsdlImportEl);
                        metadataEl.appendChild(wsdlEl);
                    }
                    else if (wsdlDefinitions != null) {
                        metadataEl.appendChild(wsdlDefinitions);
                    }
                    mexEl.appendChild(metadataEl);
                    if (msEpr.elements == null) {
                        msEpr.elements = new ArrayList();
                    }
                    msEpr.elements.add(mexEl);
                }
                else {
                    if (msEpr.elements == null) {
                        msEpr.elements = new ArrayList();
                    }
                    msEpr.elements.add(child);
                }
            }
            else if (nodes.item(i).getNodeType() == 2) {
                Node n = nodes.item(i);
                if (msEpr.attributes == null) {
                    msEpr.attributes = new HashMap();
                    String prefix = fixNull(n.getPrefix());
                    String ns = fixNull(n.getNamespaceURI());
                    String localName = n.getLocalName();
                    msEpr.attributes.put(new QName(ns, localName, prefix), n.getNodeValue());
                }
            }
        }
        return msEpr;
    }
    
    private static Map<QName, String> getAttributes(Node node) {
        Map<QName, String> attribs = null;
        NamedNodeMap nm = node.getAttributes();
        for (int i = 0; i < nm.getLength(); ++i) {
            if (attribs == null) {
                attribs = new HashMap();
            }
            Node n = nm.item(i);
            String prefix = fixNull(n.getPrefix());
            String ns = fixNull(n.getNamespaceURI());
            String localName = n.getLocalName();
            if (!prefix.equals("xmlns") && (prefix.length() != 0 || !localName.equals("xmlns")) && !localName.equals(AddressingVersion.W3C.eprType.portName)) {
                attribs.put(new QName(ns, localName, prefix), n.getNodeValue());
            }
        }
        return attribs;
    }
    
    @NotNull
    private static String fixNull(@Nullable String s) {
        if (s == null) {
            return "";
        }
        return s;
    }
    
    static {
        EndpointReferenceUtil.w3cMetadataWritten = false;
    }
}
